#version 330 core
in vec3 Normal;
in vec3 FragPos;
in vec2 TexCoords;

uniform float far_plane;
uniform vec3 lightPos;
uniform vec3 cameraPos;
uniform sampler2D texture_diffuse1;
uniform sampler2D texture_reflection1;
uniform sampler2D texture_specular1;
uniform samplerCube skybox;
uniform samplerCube depthMap;

out vec4 FragColor;

vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1), 
   vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
   vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
   vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
);

float ShadowCalculation(vec3 frag)
{
    // Get vector between fragment position and light position
    vec3 fragToLight = frag - lightPos;
    // Get current linear depth as the length between the fragment and light position
    float currentDepth = length(fragToLight);
    // Test for shadows with PCF
    float shadow = 0.0;
    float bias = 0.15;
    int samples = 20;
    float viewDistance = length(cameraPos - frag);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
    for(int i = 0; i < samples; ++i)
    {
        float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= far_plane;   // Undo mapping [0;1]
        if(currentDepth - bias > closestDepth)
            shadow += 1.0;
    }
    shadow /= float(samples);
	shadow *= 2.0f;
        
	float closestDepth = texture(depthMap, fragToLight).r;
    closestDepth *= far_plane;
    // Display closestDepth as debug (to visualize depth cubemap)
    //FragColor = vec4(vec3(closestDepth / far_plane), 1.0);    
        
    // return shadow;
    return shadow;
}



void main()
{             
    // Diffuse
    vec4 diffuse_color = texture(texture_diffuse1, TexCoords);
    // Reflection
    vec3 I = normalize(FragPos - cameraPos);
    vec3 R = reflect(I, normalize(Normal));
    float reflect_intensity = texture(texture_reflection1, TexCoords).r;
	vec4 reflect_intens = texture(texture_reflection1, TexCoords);
	vec4 reflect_col = texture(skybox, R);
    vec4 reflect_color = vec4(0.0f);
    if(reflect_intensity > 0.2) // Only sample reflections when above a certain treshold
        //reflect_color = texture(skybox, R) * reflect_intensity;
		//reflect_color = mix(reflect_intens,reflect_col,0.3);
		reflect_color = vec4(reflect_intens.r * reflect_col.r, reflect_intens.g * reflect_col.g, reflect_intens.b * reflect_col.b, reflect_intens.a * reflect_col.a); 
    // Combine them
    //vec3 color = vec3(diffuse_color + reflect_color);
	vec3 reflection = vec3(reflect_color);



	// Ambient
    vec3 ambient = 0.3 * vec3(diffuse_color);
  	
    // Diffuse 
    vec3 normal = normalize(Normal);
    vec3 lightDir = normalize(lightPos - FragPos);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 diffuse = 0.3 * diff * vec3(diffuse_color);  
    
    // Specular
    vec3 viewDir = normalize(cameraPos - FragPos);
    vec3 reflectDir = reflect(-lightDir, normal);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 16.0f);
    vec3 specular = 0.5 * spec * vec3(texture(texture_specular1, TexCoords));
        
	// Calculate shadow
    //float shadow = ShadowCalculation(FragPos); 
	float shadow = 0.0f;

    FragColor = vec4(ambient + (1.0 - shadow) * (diffuse + specular + reflection), 1.0f);  

	//FragColor = diffuse_color;

	//FragColor = vec4(1.0f);
}

